<#
  This script will enumerate all INI files in a specific location and run them against VMware's Dynamic Environment
  Manager (DEM) FlexEngine.exe to either export or import user profile settings (application data).

  This script has been tested against version 10.4 of VMware DEM, but should work with any version.

  It's recommeneded to add "FlexEngine.exe" to the "LogoffCheckSysModules" registry value to prevent a session from
  staying active should the executable hang/fail.

  This script derives its config data from the accompanying XML file. Each location has its own section. The location
  name is compared against the first x letters of the computername of the Session Host, which is used to retrieve its
  configuration data. This can easily be changed in the code to suit any naming standard.

  Example of the XML:
    <?xml version="1.0" encoding="utf-8"?>
    <!--
    <configuration>
	<Global>
		<FlexEnginePath>%ProgramFiles%\Immidio\Flex Profiles\FlexEngine.exe</FlexEnginePath>
		<DateTimeFormat>dd/MM/yyyy HH:mm:ss</DateTimeFormat>
	</Global>
	<Location Name="PER">
		<Description>Perth</Description>
		<UEMConfigShare>\\fileserver\share$\FlexProfiles\general</UEMConfigShare>
		<UEMProfileArchives>\\fileserver\share$\Profile\%username%.%userdomain%\Migration</UEMProfileArchives>
		<UEMProfileLogs>\\fileserver\share$\Profile\%username%.%userdomain%\MigrationLogs</UEMProfileLogs>
		<RecurseIniFiles>TRUE</RecurseIniFiles>
		<SplitRDSHandVDI>TRUE</SplitRDSHandVDI>
		<IncludeCommonIniFiles>TRUE</IncludeCommonIniFiles>
		<EnableDebugLogging>TRUE</EnableDebugLogging>
		<ExportFromOS>6.3.9600</ExportFromOS>
		<OnceOffExport>FALSE</OnceOffExport>
		<ExportUntil>30/06/2022 17:00:00</ExportUntil>
		<ImportToOS>10.0.19043.1165,10.0.17763.107</ImportToOS>
		<OnceOffImport>TRUE</OnceOffImport>
		<ImportUntil>30/06/2022 17:00:00</ImportUntil>
		<PlaceFlagFilesInUserProfile>TRUE</PlaceFlagFilesInUserProfile>
	</Location>
    </configuration>

    Where:
      Name = The first x letters of the computer name of the Session Hosts that help you differentiate the location so
             that the correct configuration data is used.
      Description = A description of that location.
      UEMConfigShare = The share where the ini files will be located.
      UEMProfileArchives = The per-user location where the data will be exported to and imported from.
      UEMProfileLogs = The per-user location where the FlexEngine.exe output log will be.
      RecurseIniFiles = Set to True to recurse all subfolders from the root of the UEMConfigShare location. This allows
                        you to neatly manage the ini files.
      SplitRDSHandVDI = Set to True to separate the RDSH and VDI ini files for the export/import process.
      IncludeCommonIniFiles = Set to True to include the Common ini files for the export/import process. Only valid when
                              SplitRDSHandVDI is also set to True.
      EnableDebugLogging = Set to True to enable FlexEngine.exe debug logging.
      ExportFromOS = Set the full OS version numbers (Major.Minor.Build.Revision) that you want to export from. This field
                     supports a comma separated list. Note that the Revision number is not mandatory.
      OnceOffExport = Set to True to complete a once off export for each ini file. Note that a flag file will be created
                      under the UEMProfileArchives location. The flag file will need to be deleted if you would like this
                      ini file to be processed again.
      ExportUntil = Set an end date when the exports should end. A OnceOffExport overrides this.
      ImportToOS = Set the full OS version numbers (Major.Minor.Build.Revision) that you want to import to. This field
                   supports a comma separated list. Note that the Revision number is not mandatory.
      OnceOffImport = Set to True to complete a once off import for each ini file. Note that a flag file will be created
                      under the UEMProfileArchives location. The flag file will need to be deleted if you would like this
                      ini file to be processed again.
      ImportUntil = Set an end date when the imports should end. A OnceOffImport overrides this.
      PlaceFlagFilesInUserProfile = Set to true to create the Export and Import flag files will in the user profile under
                                    the "%APPDATA%\Immidio\Flex Profiles" folder.
                                    Set to false to create the Export and Import flag files in the same location as the
                                    archives.

      If using flag files, why are we placing them on the file system instead of the registry? Whilst the registry may
      seem more efficient, it would be more difficult for Service Desk and various other Support Teams to be able to
      clear them if needed.

    Also note the Global elements:
      FlexEnginePath = The full path to the FlexEngine.exe.
      DateTimeFormat = The Date and Time format to use to convert the ExportUntil and ImportUntil XML elements.

  Syntax example:
    .\ProfileMigration.ps1 -Export -ProcessAllIniFiles -ShowUI -HideConsole
    Where:
      Import = Will instruct the script to process an import if the settings in the XML file evaluate that way.
      Export = Will instruct the script to process an export if the settings in the XML file evaluate that way.
      ProcessAllIniFiles = Will process all ini files overriding the SplitRDSHandVDI setting in the XML file.
      ShowUI = Will show the UI so that it can be used as a Desktop or Published Application.
      HideConsole = Will hide the PowerShell Console and keep the execution of the script neat.
      IgnoreFlag = Will ignore the flags created by the OnceOffImport and OnceOffExport process.
      DriveMappingsOnly = Will only process the ini files where the name starts with "DriveMappings"
   Note that Import and Export cannot be used together at the same time.

  Use Group Policy to add PowerShell Logoff and Logon Scripts:
  1) Logoff Script
     - Script Name: C:\Program Files\Immidio\Flex Profiles\ProfileMigration.ps1
     - Script Parameters: -Export
  2) Logon Script
     - Script Name: C:\Program Files\Immidio\Flex Profiles\ProfileMigration.ps1
     - Script Parameters: -Import

  Published App notes:
  1) As well as running the script at logoff and logoff, I also found it useful to publish two applications.
     This allows us to help users that may of been on leave during the migration, of cases where we may need to
     instruct users to re-export and/or re-import their profile data.
  2) Unfortunately you cannot publish the script directly as a Published App because the path to the executable
     file and command line arguments combined are greater than 203 characters. So you must use a batch script
     to publish it instead.
  Published Apps:
  1) Name: User Profile Data Export
     - Path to the executable file: C:\Program Files\Immidio\Flex Profiles\ProfileMigration-Launcher.cmd
     - Command line argument: Export
  2) Name: User Profile Data Import
     - Path to the executable file: C:\Program Files\Immidio\Flex Profiles\ProfileMigration-Launcher.cmd
     - Command line argument: Import

  Script name: ProfileMigration.ps1
  Release 2.3
  Written by Jeremy Saunders (jeremy@jhouseconsulting.com) 2nd February 2022
  Modified by Jeremy Saunders (jeremy@jhouseconsulting.com) 4th September 2023

#>

#-------------------------------------------------------------
[cmdletbinding()]
param (
       [switch]$Import,
       [switch]$Export,
       [switch]$ProcessAllIniFiles,
       [switch]$ShowUI,
       [switch]$IgnoreFlag,
       [switch]$DriveMappingsOnly,
       [switch]$HideConsole
      )

# Set Powershell Compatibility Mode
Set-StrictMode -Version 2.0

# Enable verbose, warning and error mode
$VerbosePreference = 'Continue'
$WarningPreference = 'Continue'
$ErrorPreference = 'Continue'

#-------------------------------------------------------------

$StartDTM = (Get-Date)

# Set the working folder to the users TEMP folder
$LogFolder = [System.IO.Path]::GetTempPath()

# Get the script path
$ScriptPath = {Split-Path $MyInvocation.ScriptName}
$ScriptPath = $(&$ScriptPath)

# Get the script name
$ScriptName = [System.IO.Path]::GetFilenameWithoutExtension($MyInvocation.MyCommand.Path.ToString())

# Start the transcript
try {
  Start-Transcript "$LogFolder$ScriptName.log"
}
catch {
  Write-Verbose "$(Get-Date -format "dd/MM/yyyy HH:mm:ss"): This host does not support transcription"
}

#------------------------------------

# Set the name and path for the XMLFile
$XMLFilePath = "$ScriptPath\$ScriptName.xml"

#------------------------------------

If ($Import -AND $Export) {
  write-warning "$(Get-Date): You cannot set both Import and Export to be true together." -verbose
  Exit
}

# Hide the PowerShell console window without hiding the other child windows that it spawns
# - http://powershell.cz/2013/04/04/hide-and-show-console-window-from-gui/
$Code = @"
[DllImport("Kernel32.dll")]
public static extern IntPtr GetConsoleWindow();
[DllImport("user32.dll")]
public static extern bool ShowWindow(IntPtr hWnd, Int32 nCmdShow);
"@
# Create new types as per the definition above. 
Add-Type -Namespace Console -Name Window -MemberDefinition $code -PassThru | out-null
Function Show-Console {
  $consolePtr = [Console.Window]::GetConsoleWindow()
  #5 show
  [Console.Window]::ShowWindow($consolePtr, 5)
}
Function Hide-Console {
  $consolePtr = [Console.Window]::GetConsoleWindow()
  #0 hide
  [Console.Window]::ShowWindow($consolePtr, 0)
}
If ($HideConsole) {
  Hide-Console | out-null
}

#------------------------------------

# This section is for the functions

Function EnvironmentRefresh {
# Send a WM_SETTINGCHANGE broadcast message with "Environment" as the parameter to all open windows, indicating a change in the
# environment variables. It is used to reload/refresh the environment variables in all running processes without the need for a
# system restart.
# This is used in scenarios where a change in user and system environment variables needs to be propagated immediately to all
# running applications and services.
# Notes:
# - Applications and services that have a top-level window will be notified of a change in the environment variables.
# - A restart of some applications and services may still be necessary for the changes to take effect, as some processes only
#   read environment variables at startup.
# - Not all applications will respond to the WM_SETTINGCHANGE message, especially those that do not have a top-level window or
#   those that simply choose to ignore the message. In those cases, a system restart or individual application/service restart
#   might still be necessary.
param (
       [switch]$EnableDebug
      )
$HWND_BROADCAST = [IntPtr] 0xffff;
$WM_SETTINGCHANGE = 0x1a;
$result = [UIntPtr]::Zero
if (-not ("Win32.NativeMethods" -as [Type])) {
  # import sendmessagetimeout from win32
  Add-Type -Namespace Win32 -Name NativeMethods -MemberDefinition @"
  [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
  public static extern IntPtr SendMessageTimeout(
    IntPtr hWnd, uint Msg, UIntPtr wParam, string lParam,
    uint fuFlags, uint uTimeout, out UIntPtr lpdwResult);
"@
}

# notify all windows of environment block change
If ($EnableDebug) {
  write-verbose "Sending a WM_SETTINGCHANGE message to all the open windows..." -verbose
}
[void][Win32.Nativemethods]::SendMessageTimeout($HWND_BROADCAST, $WM_SETTINGCHANGE, [UIntPtr]::Zero, "Environment", 2, 5000, [ref] $result)
If ($EnableDebug) {
  if ($result -eq 0) {
    write-warning "- Failed to reload environment variables." -verbose
  } else {
    write-verbose "- Environment variables have been reloaded." -verbose
  }
}
}

Function DesktopRefresh{
# Call the SHChangeNotify method to notify the Windows Shell to refresh or rebuild the desktop. Here we are specifically telling
# the system that a SHCNE_ASSOCCHANGED event has occurred. This event signals that a file type or association has changed. This
# is useful in situations where a change has been made to the file associations and you need the system to immediately recognize
# this change without having to log off or reboot.
   $source = @"
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
namespace FileEncryptProject.Algorithm
{
  public class DesktopRefurbish
  {
    [DllImport("shell32.dll")]
    public static extern void SHChangeNotify(HChangeNotifyEventID wEventId, HChangeNotifyFlags uFlags, IntPtr dwItem1, IntPtr dwItem2);
    public static void DeskRef()
    {
      SHChangeNotify(HChangeNotifyEventID.SHCNE_ASSOCCHANGED, HChangeNotifyFlags.SHCNF_IDLIST, IntPtr.Zero, IntPtr.Zero);
    }
  }
  #region public enum HChangeNotifyFlags
  [Flags]
  public enum HChangeNotifyFlags
  {
    SHCNF_DWORD = 0x0003,
    SHCNF_IDLIST = 0x0000,
    SHCNF_PATHA = 0x0001,
    SHCNF_PATHW = 0x0005,
    SHCNF_PRINTERA = 0x0002,
    SHCNF_PRINTERW = 0x0006,
    SHCNF_FLUSH = 0x1000,
    SHCNF_FLUSHNOWAIT = 0x2000
  }
  #endregion//enum HChangeNotifyFlags
  #region enum HChangeNotifyEventID
  [Flags]
  public enum HChangeNotifyEventID
  {
    SHCNE_ALLEVENTS = 0x7FFFFFFF,
    SHCNE_ASSOCCHANGED = 0x08000000,
    SHCNE_ATTRIBUTES = 0x00000800,
    SHCNE_CREATE = 0x00000002,
    SHCNE_DELETE = 0x00000004,
    SHCNE_DRIVEADD = 0x00000100,
    SHCNE_DRIVEADDGUI = 0x00010000,
    SHCNE_DRIVEREMOVED = 0x00000080,
    SHCNE_EXTENDED_EVENT = 0x04000000,
    SHCNE_FREESPACE = 0x00040000,
    SHCNE_MEDIAINSERTED = 0x00000020,
    SHCNE_MEDIAREMOVED = 0x00000040,
    SHCNE_MKDIR = 0x00000008,
    SHCNE_NETSHARE = 0x00000200,
    SHCNE_NETUNSHARE = 0x00000400,
    SHCNE_RENAMEFOLDER = 0x00020000,
    SHCNE_RENAMEITEM = 0x00000001,
    SHCNE_RMDIR = 0x00000010,
    SHCNE_SERVERDISCONNECT = 0x00004000,
    SHCNE_UPDATEDIR = 0x00001000,
    SHCNE_UPDATEIMAGE = 0x00008000,
  }
  #endregion
}
"@
     Add-Type -TypeDefinition $source
    [FileEncryptProject.Algorithm.DesktopRefurbish]::DeskRef()
}

Function RemapUnavailableNetworkDrives {
  # Remap any unavailable network drives that have a red X against them
  # Reference: https://docs.microsoft.com/en-us/troubleshoot/windows-client/networking/mapped-network-drive-fail-reconnect
  # Can re-write this with "net use" easily enough if needed
  $i=3
  while($True){
    $error.clear()
    $MappedDrives = Get-SmbMapping | where -property Status -Value Unavailable -EQ | select LocalPath,RemotePath
    If (($MappedDrives | Measure-Object).Count -gt 0) {
      Write-verbose "$(Get-Date): Remapping Unavailable Network Drives" -verbose
      foreach( $MappedDrive in $MappedDrives) {
        try {
          Remove-SmbMapping -LocalPath $MappedDrive.LocalPath -UpdateProfile -Force -ErrorAction SilentlyContinue
          Write-verbose "$(Get-Date): - Mapping $($MappedDrive.LocalPath) to $($MappedDrive.RemotePath)" -verbose
          New-SmbMapping -LocalPath $MappedDrive.LocalPath -RemotePath $MappedDrive.RemotePath -Persistent $True
        } catch {
          Write-warning "$(Get-Date): - There was an error mapping $($MappedDrive.LocalPath) to $($MappedDrive.RemotePath)" -verbose
        }
      }
    }
    $i = $i - 1
    if($error.Count -eq 0 -Or $i -eq 0) {break}
    Start-Sleep -Seconds 1
  }
  $FinalMappedDrives = Get-SmbMapping | where -property Status -Value OK -EQ | select LocalPath,RemotePath
  If (($FinalMappedDrives | Measure-Object).Count -gt 0) {
    Write-verbose "$(Get-Date): Mapped Network Drives" -verbose
    foreach($FinalMappedDrive in $FinalMappedDrives) {
      Write-verbose "$(Get-Date): - $($FinalMappedDrive.LocalPath) = $($FinalMappedDrive.RemotePath)" -verbose
    }
  }
}

Function RestartProcessUsingRestartManager {
# This PowerShell function is derived from C# code written by Vanco Pavlevski and published in his article titled "Restart Explorer
# Programmatically with C#" (https://devindeep.com/restart-explorer-programmatically-with-c/)
# It uses the Restart Manager API to restart a process. The code has been modified and optimised to work on a multi-session hosts
# so it will only restart the process in the current user session.
# The benefit of restarting Explorer this way is that Explorer knows how to restart itself, and it will restore all existing open
# windows after the restart. Therefore it's user friendly. This is also a really nice way of restarting the Explorer shell without
# requiring the user to log off and back on again for specific changes.
param (
       [string]$Name
      )

Add-Type -TypeDefinition @"
using System;
using System.Collections.Generic;
using System.Diagnostics;
using Com = System.Runtime.InteropServices.ComTypes;
using System.Runtime.InteropServices;

public static class RestartManager
{
    [StructLayout(LayoutKind.Sequential)]
    struct RM_UNIQUE_PROCESS
    {
        public int dwProcessId;
        public Com.FILETIME ProcessStartTime;
    }

    [Flags]
    enum RM_SHUTDOWN_TYPE : uint
    {
        RmForceShutdown = 0x1,
        RmShutdownOnlyRegistered = 0x10
    }

    delegate void RM_WRITE_STATUS_CALLBACK(UInt32 nPercentComplete);

    [DllImport("rstrtmgr.dll", CharSet = CharSet.Auto)]
    static extern int RmStartSession(out IntPtr pSessionHandle, int dwSessionFlags, string strSessionKey);

    [DllImport("rstrtmgr.dll")]
    static extern int RmEndSession(IntPtr pSessionHandle);

    [DllImport("rstrtmgr.dll", CharSet = CharSet.Auto)]
    static extern int RmRegisterResources(IntPtr pSessionHandle, UInt32 nFiles, string[] rgsFilenames, UInt32 nApplications, RM_UNIQUE_PROCESS[] rgApplications, UInt32 nServices, string[] rgsServiceNames);

    [DllImport("rstrtmgr.dll")]
    static extern int RmShutdown(IntPtr pSessionHandle, RM_SHUTDOWN_TYPE lActionFlags, RM_WRITE_STATUS_CALLBACK fnStatus);

    [DllImport("rstrtmgr.dll")]
    static extern int RmRestart(IntPtr pSessionHandle, int dwRestartFlags, RM_WRITE_STATUS_CALLBACK fnStatus);

    [DllImport("kernel32.dll")]
    static extern bool GetProcessTimes(IntPtr hProcess, out Com.FILETIME lpCreationTime, out Com.FILETIME lpExitTime, out Com.FILETIME lpKernelTime, out Com.FILETIME lpUserTime);

    public static void RestartProcessInCurrentSession(string process)
    {
        if (!string.IsNullOrEmpty(process))
        {
            lock (typeof(RestartManager))
            {
                int waitTime = 2000;
                IntPtr handle;
                string key = Guid.NewGuid().ToString();
                int sessionID = Process.GetCurrentProcess().SessionId;
                int res = RmStartSession(out handle, 0, key);
                if (res == 0)
                {
                    Console.WriteLine("Restart Manager session created with ID " + key);
                    Console.WriteLine("Checking for " + process + " processes running under Session ID " + sessionID.ToString());
                    List<RM_UNIQUE_PROCESS> lst = new List<RM_UNIQUE_PROCESS>();
                    foreach (Process p in Process.GetProcessesByName(process))
                    {
                        if (p.SessionId == sessionID)
                        {
                            RM_UNIQUE_PROCESS rp = new RM_UNIQUE_PROCESS();
                            rp.dwProcessId = p.Id;
                            Com.FILETIME creationTime, exitTime, kernelTime, userTime;
                            GetProcessTimes(p.Handle, out creationTime, out exitTime, out kernelTime, out userTime);
                            rp.ProcessStartTime = creationTime;
                            lst.Add(rp);
                        }
                    }
                    RM_UNIQUE_PROCESS[] processes = lst.ToArray();
                    if (processes.Length > 0)
                    {
                        Console.WriteLine(processes.Length + " unique " + process + " process found.");
                        res = RmRegisterResources(
                            handle,
                            0, null,
                            (uint)processes.Length, processes,
                            0, null
                        );
                        if (res == 0)
                        {
                            Console.WriteLine("Successfully registered resources.");
                            Console.WriteLine("Stopping the " + process + " process...");
                            res = RmShutdown(handle, RM_SHUTDOWN_TYPE.RmForceShutdown, null);
                            if (res == 0)
                            {
                                Console.WriteLine("The " + process + " process stopped successfully.");
                                System.Threading.Thread.Sleep(waitTime);
                                Console.WriteLine("Restarting the " + process + " process...");
                                res = RmRestart(handle, 0, null);
                                if (res == 0)
                                {
                                    Console.WriteLine("The " + process + " process restarted successfully.");
                                }
                            }
                        }
                    }
                    else
                    {
                        Console.WriteLine("There are no " + process + " processes running.");
                    }
                    res = RmEndSession(handle);
                    if (res == 0)
                    {
                        Console.WriteLine("Restart Manager session ended.");
                    }
                }
            }
        }
        else
        {
            Console.WriteLine("The process name was not specified.");
        }
    }
}
"@
  $Result = [RestartManager]::RestartProcessInCurrentSession($Name)
}

Function Call-Main {
  # This is for the main program
  param (
         [switch]$Import,
         [switch]$Export,
         [switch]$ProcessAllIniFiles,
         [switch]$ShowUI,
         [switch]$IgnoreFlag,
         [switch]$DriveMappingsOnly
        )

  $StatusMessage = ""

  If (Test-Path -path $XMLFilePath) {

    # Create the XML Object and open the XML file
    [xml]$XmlDocument = Get-Content -path $XMLFilePath

    # Uncomment the following lines for debugging purposes only
    #$XmlDocument.configuration.Global
    #$XmlDocument.configuration.Global | Format-Table -Autosize

    foreach ($Config in $XmlDocument.configuration.Global) {
      $FlexEnginePath = $Config.FlexEnginePath
      $FlexEnginePath = $FlexEnginePath.Replace("%ProgramFiles(x86)%",${env:ProgramFiles(x86)})
      $FlexEnginePath = $FlexEnginePath.Replace("%ProgramFiles%",${env:ProgramFiles})
      $FlexEnginePath = $FlexEnginePath.Replace("%ProgramData%",${env:ProgramData})
      $FlexEnginePath = $FlexEnginePath.Replace("%SystemDrive%",${env:SystemDrive})
      $FlexEnginePath = $FlexEnginePath.Replace("%SystemRoot%",${env:SystemRoot})
      $DateTimeFormat = $Config.DateTimeFormat
    }

    Write-verbose "$(Get-Date): Global variables" -verbose
    Write-verbose "$(Get-Date): - FlexEnginePath: $FlexEnginePath" -verbose
    Write-verbose "$(Get-Date): - DateTimeFormat: $DateTimeFormat" -verbose

    # Uncomment the following lines for debugging purposes only
    #$XmlDocument.configuration.Location
    #$XmlDocument.configuration.Location | Format-Table -Autosize

    $MappedDrives = $False
    $ConfigFound = $False

    foreach ($Config in $XmlDocument.configuration.Location) {
      If ($env:COMPUTERNAME -Like "$($Config.Name)*") {
        $ConfigFound = $True
        $Description = $Config.Description
        $UEMConfigShare = $Config.UEMConfigShare
        $UEMProfileArchives = $Config.UEMProfileArchives
        $UEMProfileArchives = $UEMProfileArchives.Replace("%username%",${env:username})
        $UEMProfileArchives = $UEMProfileArchives.Replace("%userdomain%",${env:userdomain})
        $UEMProfileLogs = $Config.UEMProfileLogs
        $UEMProfileLogs = $UEMProfileLogs.Replace("%username%",${env:username})
        $UEMProfileLogs = $UEMProfileLogs.Replace("%userdomain%",${env:userdomain})
        $RecurseIniFiles = [System.Convert]::ToBoolean($Config.RecurseIniFiles)
        $SplitRDSHandVDI = [System.Convert]::ToBoolean($Config.SplitRDSHandVDI)
        $IncludeCommonIniFiles = [System.Convert]::ToBoolean($Config.IncludeCommonIniFiles)
        $EnableDebugLogging = [System.Convert]::ToBoolean($Config.EnableDebugLogging)
        $ExportFromOS = $Config.ExportFromOS
        $OnceOffExport = [System.Convert]::ToBoolean($Config.OnceOffExport)
        If ([String]::IsNullOrEmpty($Config.ExportUntil)) {
          $ExportUntil = [Datetime]::ParseExact(((Get-Date).AddMinutes(10) | Get-Date -Format $DateTimeFormat), $DateTimeFormat, $null)
        } Else {
          Try {
            $ExportUntil = [Datetime]::ParseExact($Config.ExportUntil, $DateTimeFormat, $null)
          }
          Catch {
            $ExportUntil = [Datetime]::ParseExact(((Get-Date).AddMinutes(10) | Get-Date -Format $DateTimeFormat), $DateTimeFormat, $null)
          }
        }
        $ImportToOS = $Config.ImportToOS
        $OnceOffImport = [System.Convert]::ToBoolean($Config.OnceOffImport)
        If ([String]::IsNullOrEmpty($Config.ImportUntil)) {
          $ImportUntil = [Datetime]::ParseExact(((Get-Date).AddMinutes(10) | Get-Date -Format $DateTimeFormat), $DateTimeFormat, $null)
        } Else {
          Try {
            $ImportUntil = [Datetime]::ParseExact($Config.ImportUntil, $DateTimeFormat, $null)
          }
          Catch {
            $ImportUntil = [Datetime]::ParseExact(((Get-Date).AddMinutes(10) | Get-Date -Format $DateTimeFormat), $DateTimeFormat, $null)
          }
        }
        $PlaceFlagFilesInUserProfile = [System.Convert]::ToBoolean($Config.PlaceFlagFilesInUserProfile)

        $StatusMessage = "Found the settings for $Description"
        If ($ShowUI) {
          $status1TextBox.AppendText("$StatusMessage `r`n")
          $status1TextBox.ScrollToCaret()
        }
        Write-verbose "$(Get-Date): $StatusMessage" -verbose
        Write-verbose "$(Get-Date): - UEMConfigShare: $UEMConfigShare" -verbose
        Write-verbose "$(Get-Date): - UEMProfileArchives: $UEMProfileArchives" -verbose
        Write-verbose "$(Get-Date): - UEMProfileLogs: $UEMProfileLogs" -verbose
        Write-verbose "$(Get-Date): - RecurseIniFiles: $RecurseIniFiles" -verbose
        Write-verbose "$(Get-Date): - SplitRDSHandVDI: $SplitRDSHandVDI" -verbose
        Write-verbose "$(Get-Date): - IncludeCommonIniFiles: $IncludeCommonIniFiles" -verbose
        Write-verbose "$(Get-Date): - EnableDebugLogging: $EnableDebugLogging" -verbose
        Write-verbose "$(Get-Date): - ExportFromOS: $ExportFromOS" -verbose
        Write-verbose "$(Get-Date): - OnceOffExport: $OnceOffExport" -verbose
        Write-verbose "$(Get-Date): - ExportUntil: $ExportUntil" -verbose
        Write-verbose "$(Get-Date): - ImportToOS: $ImportToOS" -verbose
        Write-verbose "$(Get-Date): - OnceOffImport: $OnceOffImport" -verbose
        Write-verbose "$(Get-Date): - ImportUntil: $ImportUntil" -verbose
        Write-verbose "$(Get-Date): - PlaceFlagFilesInUserProfile: $PlaceFlagFilesInUserProfile" -verbose
      }
    }

    If ($ConfigFound) {

      If (Test-Path -path "$FlexEnginePath") {

        Write-verbose "$(Get-Date): Script parameters:" -verbose
        Write-verbose "$(Get-Date): - Import: $Import" -verbose
        Write-verbose "$(Get-Date): - Export: $Export" -verbose
        Write-verbose "$(Get-Date): - ProcessAllIniFiles: $ProcessAllIniFiles" -verbose
        Write-verbose "$(Get-Date): - ShowUI: $ShowUI" -verbose
        Write-verbose "$(Get-Date): - IgnoreFlag: $IgnoreFlag" -verbose
        Write-verbose "$(Get-Date): - DriveMappingsOnly: $DriveMappingsOnly" -verbose

        Write-verbose "$(Get-Date): Action to be taken:" -verbose

        # Get the Operating System Major, Minor, Build and Revision Version Numbers
        $OSVersion = [System.Environment]::OSVersion.Version
        [int]$OSMajorVer = $OSVersion.Major
        [int]$OSMinorVer = $OSVersion.Minor
        [int]$OSBuildVer = $OSVersion.Build
        # Get the UBR (Update Build Revision) from the registry so that the full build number
        # is the same as the output of of the ver command line.
        [int]$OSRevisionVer = 0
        $Path = "HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion"
        $ValueExists = $False
        $ErrorActionPreference = "stop"
        try {
          If ((Get-ItemProperty -Path "$Path" | Select-Object -ExpandProperty "UBR") -ne $null) {
            $ValueExists = $True
          }
        }
        catch {
          #
        }
        $ErrorActionPreference = "Continue"
        If ($ValueExists) {
          [int]$OSRevisionVer = (Get-ItemProperty -Path "$Path" -Name UBR).UBR
        }

        # Get Operating System information so we can split based on ProductType.
        $OperatingSystem = Get-WmiObject Win32_OperatingSystem | Select-Object ProductType

        Write-verbose "$(Get-Date): - RecurseIniFiles: $RecurseIniFiles" -verbose
        If ($ProcessAllIniFiles -eq $False) {
          Write-verbose "$(Get-Date): - SplitRDSHandVDI: $SplitRDSHandVDI" -verbose
          Write-verbose "$(Get-Date): - IncludeCommonIniFiles: $IncludeCommonIniFiles" -verbose
        } Else {
          $SplitRDSHandVDI = $False
          $IncludeCommonIniFiles = $False
          Write-verbose "$(Get-Date): - ProcessAllIniFiles: $ProcessAllIniFiles" -verbose
        }
        Write-verbose "$(Get-Date): - EnableDebugLogging: $EnableDebugLogging" -verbose

        $MachineType = ""
        If ($OperatingSystem.ProductType -eq 1) { $MachineType = "VDI" }
        If ($OperatingSystem.ProductType -eq 3) { $MachineType = "RDSH" }
        Write-verbose "$(Get-Date): - MachineType: $MachineType" -verbose

        $ProcessExport = $False
        ForEach ($ExportOS in $ExportFromOS.Split(",")) {
          [int]$Count = ($ExportOS.Split(".") | Measure-Object).Count
          If ($Count -eq 3) {
            If ($OSMajorVer -eq [int]$ExportOS.Split(".")[0] -AND $OSMinorVer -eq [int]$ExportOS.Split(".")[1] -AND $OSBuildVer -eq [int]$ExportOS.Split(".")[2]) {
              $ProcessExport = $True
              Break
            }
          }
          If ($Count -eq 4) {
            If ($OSMajorVer -eq [int]$ExportOS.Split(".")[0] -AND $OSMinorVer -eq [int]$ExportOS.Split(".")[1] -AND $OSBuildVer -eq [int]$ExportOS.Split(".")[2] -AND $OSRevisionVer -eq [int]$ExportOS.Split(".")[3]) {
              $ProcessExport = $True
              Break
            }
          }
        }
        If ($ProcessExport -AND $Export) {
          Write-verbose "$(Get-Date): - Exporting" -verbose
          Write-verbose "$(Get-Date): Assessing the Export process:" -verbose
          If ((Get-Date) -le $ExportUntil) {
            Write-verbose "$(Get-Date): - Date range is valid" -verbose
          } Else {
            $StatusMessage = "Date range has expired. The Export will not run again unless the date range is extended."
            If ($ShowUI) {
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
            }
            Write-verbose "$(Get-Date): - $StatusMessage" -verbose
            $ProcessExport = $False
          }
          Write-verbose "$(Get-Date): - Exporting: $ProcessExport" -verbose
        }

        $ProcessImport = $False
        ForEach ($ImportOS in $ImportToOS.Split(",")) {
          [int]$Count = ($ImportOS.Split(".") | Measure-Object).Count
          If ($Count -eq 3) {
            If ($OSMajorVer -eq [int]$ImportOS.Split(".")[0] -AND $OSMinorVer -eq [int]$ImportOS.Split(".")[1] -AND $OSBuildVer -eq [int]$ImportOS.Split(".")[2]) {
              $ProcessImport = $True
              Break
            }
          }
          If ($Count -eq 4) {
            If ($OSMajorVer -eq [int]$ImportOS.Split(".")[0] -AND $OSMinorVer -eq [int]$ImportOS.Split(".")[1] -AND $OSBuildVer -eq [int]$ImportOS.Split(".")[2] -AND $OSRevisionVer -eq [int]$ImportOS.Split(".")[3]) {
              $ProcessImport = $True
              Break
            }
          }
        }
        If ($ProcessImport -AND $Import) {
          Write-verbose "$(Get-Date): - Importing" -verbose
          Write-verbose "$(Get-Date): Assessing the Import process:" -verbose
          If ((Get-Date) -le $ImportUntil) {
            Write-verbose "$(Get-Date): - Date range is valid" -verbose
          } Else {
            $StatusMessage = "Date range has expired. The Import will not run again unless the date range is extended."
            If ($ShowUI) {
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
            }
            Write-verbose "$(Get-Date): - $StatusMessage" -verbose
            $ProcessImport = $False
          }
          Write-verbose "$(Get-Date): - Importing: $ProcessImport" -verbose
        }

        $OutputLog = "$UEMProfileLogs\FlexEngine.log"
        If ($DriveMappingsOnly -eq $False) {
          If ($SplitRDSHandVDI) {
            $IniFiles = Get-ChildItem "$UEMConfigShare\$MachineType" -recurse:$RecurseIniFiles -ErrorAction SilentlyContinue
            If ($IncludeCommonIniFiles) {
              $IniFiles += Get-ChildItem "$UEMConfigShare\Common" -recurse:$RecurseIniFiles -ErrorAction SilentlyContinue
            }
            $OutputLog = "$UEMProfileLogs\FlexEngine" + $MachineType + ".log"
          } Else {
            $IniFiles = Get-ChildItem "$UEMConfigShare" -recurse:$RecurseIniFiles -ErrorAction SilentlyContinue
          }
        } Else {
          $IniFiles = Get-ChildItem "$UEMConfigShare" -recurse:$True -ErrorAction SilentlyContinue | where {$_.Name -like "MappedDrives*"}
        }

        If (($ProcessExport -AND $Export) -OR ($ProcessImport -AND $Import)) {
          If (($IniFiles | Measure-Object).Count -gt 0) {
            $IniFiles | ForEach-Object {
              If ($_.Extension -eq ".INI") {
                $FullPathofINI = $_.FullName
                $DirectoryNameofINI = $_.DirectoryName
                If ($UEMConfigShare.Length -eq $DirectoryNameofINI.Length) {
                  $FullPathofZIP = $UEMProfileArchives + "\" + ($_.Name.Replace('.INI','.zip'))
                } Else {
                  $FullPathofZIP = $UEMProfileArchives + $DirectoryNameofINI.Substring($UEMConfigShare.Length, $DirectoryNameofINI.Length - $UEMConfigShare.Length) + "\" + ($_.Name.Replace('.INI','.zip'))
                }
                If ($PlaceFlagFilesInUserProfile) {
                  $RunOnceImportFlag = "${env:AppData}\Immidio\Flex Profiles" + "\" + ($_.Name.Replace('.INI','RunOnceImportFlag'))
                  $RunOnceExportFlag = "${env:AppData}\Immidio\Flex Profiles" + "\" + ($_.Name.Replace('.INI','RunOnceExportFlag'))
                } Else {
                  $RunOnceImportFlag = $UEMProfileArchives + "\" + ($_.Name.Replace('.INI','RunOnceImportFlag'))
                  $RunOnceExportFlag = $UEMProfileArchives + "\" + ($_.Name.Replace('.INI','RunOnceExportFlag'))
                }
                $ProcessThisFile = $True

                $StatusMessage = "Processing $($_.Basename) settings..."
                If ($ShowUI) {
                  $status1TextBox.AppendText("$StatusMessage `r`n")
                  $status1TextBox.ScrollToCaret()
                }

                Write-verbose "$(Get-Date): Processing `"$FullPathofINI`"" -verbose
  
                If ($IgnoreFlag) {
                  $OnceOffImport = $False
                  $OnceOffExport = $False
                }

                If ($ProcessImport -AND $OnceOffImport) {
                  If (!(Test-Path -Path "$RunOnceImportFlag")) {
                    Write-verbose "$(Get-Date): - The run once import flag does not exist. It will be processed." -verbose
                  } Else {
                    $StatusMessage = "- The import has previously been completed. It will not run again."
                    If ($ShowUI) {
                      $status1TextBox.AppendText("$StatusMessage `r`n")
                      $status1TextBox.ScrollToCaret()
                    }
                    Write-verbose "$(Get-Date): $StatusMessage " -verbose
                    $ProcessThisFile = $False
                  }
                }

                If ($ProcessExport -AND $OnceOffExport) {
                  If (!(Test-Path -Path "$RunOnceExportFlag")) {
                    Write-verbose "$(Get-Date): - The run once export flag does not exist. It will be processed." -verbose
                  } Else {
                    $StatusMessage = "- The export has previously been completed. It will not run again."
                    If ($ShowUI) {
                      $status1TextBox.AppendText("$StatusMessage `r`n")
                      $status1TextBox.ScrollToCaret()
                    }
                    Write-verbose "$(Get-Date): $StatusMessage " -verbose
                    $ProcessThisFile = $False
                  }
                }

                If ($ProcessImport -AND $Import) {
                  If (!(Test-Path -Path "$FullPathofZIP")) {
                    $StatusMessage = "- A corresponding archive (zip) does not exist, so the import will not run."
                    If ($ShowUI) {
                      $status1TextBox.AppendText("$StatusMessage `r`n")
                      $status1TextBox.ScrollToCaret()
                    }
                    Write-verbose "$(Get-Date): $StatusMessage " -verbose
                    $ProcessThisFile = $False
                  }
                }

                If ($ProcessThisFile) {

                  If ($_.Name -Like "MappedDrives*" ) {
                    $MappedDrives = $True
                  }

                  $CmdLineArgs = @()
                  # Although the -i parameter is not applicable for file-based imports, it can still be left in the
                  # arguments passed and will be ignored. We use it in the code above for the import process to ensure
                  # there is a matching archive (zip) for each ini file.
                  $CmdLineArgs += "-i"
                  $CmdLineArgs += "`"$FullPathofINI`""
                  If ($ProcessImport) { $CmdLineArgs += "-r" }
                  If ($ProcessExport) { $CmdLineArgs += "-s" }
                  $CmdLineArgs += "`"$FullPathofZIP`""
                  # The -b parameter is not used for imports
                  If ($ProcessExport) { $CmdLineArgs += "-b-" }
                  $CmdLineArgs += "-R"
                  $CmdLineArgs += "-f"
                  $CmdLineArgs += "`"$OutputLog`""
                  If ($EnableDebugLogging) {
                    $CmdLineArgs += "-l"
                    $CmdLineArgs += "DEBUG"
                  }

                  $Success = $False
                  $pinfo = New-Object System.Diagnostics.ProcessStartInfo
                  $pinfo.FileName = $FlexEnginePath
                  $pinfo.UseShellExecute = $false
                  $pinfo.RedirectStandardOutput = $true
                  $pinfo.RedirectStandardError = $true
                  $pinfo.Arguments = $CmdLineArgs
                  $p = New-Object System.Diagnostics.Process
                  $p.StartInfo = $pinfo
                  Try {
                    $p.Start() | Out-Null
                    $p.WaitForExit()
                    $output = $p.StandardOutput.ReadToEnd()
                    $output += $p.StandardError.ReadToEnd()
                    #$output
                    If ($p.ExitCode -eq 0) {
                      $Success = $True
                      Write-verbose "$(Get-Date): Successfully executed" -Verbose
                      Write-verbose "$(Get-Date): Exit code: $($p.ExitCode)" -verbose
                    } Else {
                      Write-warning "$(Get-Date): Failed to execute" -Verbose
                      Write-warning "$(Get-Date): Exit code: $($p.ExitCode)" -verbose
                    }
                  }
                  Catch {
                    write-warning "$(Get-Date): Failed to execute" -Verbose
                  }
                  If ($Success) {
                    If ($ProcessImport -AND $OnceOffImport) {
                      Write-verbose "$(Get-Date): Creating the Import Flag so that it doesn't import again" -verbose
                      $ParentFolder = (Split-Path -path $RunOnceImportFlag -parent)
                      If (!(Test-Path -Path $ParentFolder)) {
                        New-Item $ParentFolder -itemtype directory | out-null
                      }
                      Out-File -FilePath "$RunOnceImportFlag"
                    }
                    If ($ProcessExport -AND $OnceOffExport) {
                      Write-verbose "$(Get-Date): Creating the Export Flag so that it doesn't export again" -verbose
                      $ParentFolder = (Split-Path -path $RunOnceExportFlag -parent)
                      If (!(Test-Path -Path $ParentFolder)) {
                        New-Item $ParentFolder -itemtype directory | out-null
                      }
                      Out-File -FilePath "$RunOnceExportFlag"
                    }
                  }
                  $p.Dispose()
                } Else {
                  Write-verbose "$(Get-Date): - No action will be taken" -verbose
                }
              }
            }
          } Else {
            Write-verbose "$(Get-Date): No ini files were found" -verbose
          }
        } Else {
          If ($Import) {
            $StatusMessage = "A valid Import profile was not found for this Operating System version."
            If ($ShowUI) {
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
            }
            write-verbose "$(Get-Date): $StatusMessage" -verbose
          }
          If ($Export) {
            $StatusMessage = "A valid Export profile was not found for this Operating System version."
            If ($ShowUI) {
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
            }
            write-verbose "$(Get-Date): $StatusMessage" -verbose
          }
          If ($Import -eq $False -AND $Export -eq $False) {
            $StatusMessage = "No Import or Export action will be taken."
            If ($ShowUI) {
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
            }
            write-verbose "$(Get-Date): $StatusMessage" -verbose
          }
        }

        If ($MappedDrives -AND $ProcessImport -AND $Import) {
          $StatusMessage = "Refreshing mapped drives..."
          If ($ShowUI) {
            # On the odd occasion we may get an error here saying that the "Method invocation failed because
            # [System.String] does not contain a method named 'Length'". So I've wrapped this in a try/catch.
            Try {
              $TextLength = $status1TextBox.Text.Length
              $status1TextBox.AppendText("$StatusMessage `r`n")
              $status1TextBox.ScrollToCaret()
              # Sleep to allow the UI message to update before Explorer.exe is terminated.
              do { 
                  Start-sleep -s 1
                 }  
              until ($status1TextBox.Text.Length -eq $TextLength + $StatusMessage.Length())
            }
            catch {
              #
            }
          }
          write-verbose "$(Get-Date): $StatusMessage" -verbose
          RemapUnavailableNetworkDrives
          RestartProcessUsingRestartManager -Name:"explorer"
        }
        If ($ProcessImport -AND $Import -AND $DriveMappingsOnly -eq $False) {
          $StatusMessage = "Refreshing environment variables and file associations..."
          If ($ShowUI) {
            $status1TextBox.AppendText("$StatusMessage `r`n")
            $status1TextBox.ScrollToCaret()
          }
          write-verbose "$(Get-Date): $StatusMessage" -verbose
          EnvironmentRefresh
          DesktopRefresh
        }

        $StatusMessage = "Finished!"
        If ($ShowUI) {
          $status1TextBox.AppendText("$StatusMessage `r`n")
          $status1TextBox.ScrollToCaret()
        }
        write-verbose "$(Get-Date): $StatusMessage" -verbose

      } Else {
        Write-warning "$(Get-Date): `"$FlexEnginePath`" does not exist" -verbose
      }

    } Else {
      write-warning "$(Get-Date): No valid config found" -verbose
    }

  } Else {
    write-warning "$(Get-Date): The XML file was not found" -verbose
  }
}

Function Call-MainForm {
  # This is for the user form
  param (
         [switch]$Import,
         [switch]$Export,
         [switch]$ProcessAllIniFiles,
         [switch]$ShowUI,
         [switch]$IgnoreFlag,
         [switch]$DriveMappingsOnly
        )

$codeDisableX = @"
using System.Windows.Forms;

namespace MyForm {
    public class FormWithoutX : Form {
        protected override CreateParams CreateParams {
            get {
                CreateParams cp = base.CreateParams;
                cp.ClassStyle = cp.ClassStyle | 0x200;
                return cp;
            }
        }
    }
}
"@

  Add-Type -AssemblyName System.Windows.Forms
  Add-Type -TypeDefinition $codeDisableX -ReferencedAssemblies System.Windows.Forms
  Add-Type -AssemblyName System.Drawing
  Add-Type -AssemblyName PresentationCore,PresentationFramework

  $IconFile = "$ScriptPath\File Export.ico"
  If ($Import) {
    $IconFile = "$ScriptPath\File Import.ico"
  }

  $form = [MyForm.FormWithoutX]::new()
  #$form = New-Object System.Windows.Forms.Form
  $form.Text = 'User Profile Migration Tool'
  $form.ClientSize = New-Object System.Drawing.Size(350,350)
  $form.MaximizeBox = $False
  $form.MinimizeBox = $False
  $form.MinimumSize = New-Object System.Drawing.Size(350,440)
  $form.MaximumSize = New-Object System.Drawing.Size(350,440)
  $form.FormBorderStyle = "FixedDialog"
  $form.SizeGripStyle = "Hide"
  If (Test-Path "$IconFile") {
    $form.Icon = [System.Drawing.Icon]::ExtractAssociatedIcon("$IconFile")
  }
  $form.WindowState = 'Normal'

  # Start the form from the top left eighth position of the primary screen. This ensures
  # it starts consistently in the same location and out of the way of any popups.
  $LocationWidth = $(([System.Windows.Forms.Screen]::PrimaryScreen.WorkingArea.Width /8))
  $LocationHeight = $(([System.Windows.Forms.Screen]::PrimaryScreen.WorkingArea.Height /8))
  $form.Location = New-Object System.Drawing.Point($LocationWidth,$LocationHeight)
  $form.StartPosition = 'Manual'

  $labelheader = New-Object 'System.Windows.Forms.Label'
  $labelheader.Font = 'Microsoft Sans Serif, 14.25pt'
  $labelheader.ForeColor = 'White'
  $labelheader.Location = New-Object System.Drawing.Point(47, 18)
  $labelheader.Name = 'labelheader'
  $labelheader.Size = New-Object System.Drawing.Size(350, 23)
  $labelheader.TabIndex = 1
  $labelheader.Text = 'User Profile Migration Tool'
  $labelheader.TextAlign = 'MiddleLeft'

  $panelheader = New-Object 'System.Windows.Forms.Panel'
  $panelheader.Controls.Add($labelheader)
  $panelheader.BackColor = '0, 114, 198'
  $panelheader.Location = New-Object System.Drawing.Point(0, 0)
  $panelheader.Name = 'header'
  $panelheader.Size = New-Object System.Drawing.Size(350, 67)
  $panelheader.TabIndex = 8
  $form.Controls.Add($panelheader)

  $closeButton = New-Object System.Windows.Forms.Button
  $closeButton.Location = New-Object System.Drawing.Point(250,210)
  $closeButton.Size = New-Object System.Drawing.Size(75,23)
  $closeButton.Text = 'Close'
  $closeButton.DialogResult = [System.Windows.Forms.DialogResult]::OK
  $form.CancelButton = $closeButton
  $form.Controls.Add($closeButton)

  $status1label = New-Object System.Windows.Forms.Label
  $status1label.Font = 'Microsoft Sans Serif, 9pt'
  $status1label.Location = New-Object System.Drawing.Point(10,215)
  $status1label.Name = 'status1label'
  $status1label.Size = New-Object System.Drawing.Size(100,23)
  $status1label.UseMnemonic = $false
  $status1label.Text = "Output message:"
  $form.Controls.Add($status1label)

  $status1TextBox = New-Object System.Windows.Forms.RichTextBox
  $status1TextBox.Location = New-Object System.Drawing.Point(10,240)
  $status1TextBox.Size = New-Object System.Drawing.Size(315, 150)
  $status1TextBox.Multiline = $true
  $status1TextBox.WordWrap = $true
  $status1TextBox.ScrollBars = [System.Windows.Forms.ScrollBars]::'Vertical'
  $status1TextBox.Enabled = $true
  $status1TextBox.ReadOnly = $true
  $oldFont =  $status1label.Font
  $newFont = New-Object System.Drawing.Font($oldFont.FontFamily, $oldFont.Size, [System.Drawing.FontStyle]::Bold)
  $status1TextBox.Font = $newFont
  $form.Controls.Add($status1TextBox)

  #-------------------------------------------------------------

  $action1Button = New-Object System.Windows.Forms.Button
  $action1Button.Location = New-Object System.Drawing.Point(90,80)
  $action1Button.Size = New-Object System.Drawing.Size(150,23)

  If ($Export) {
    $buttonText1 = "Export my settings"

    $GroupBox1 = New-Object System.Windows.Forms.GroupBox
    $GroupBox1.Location = '40,110'
    $GroupBox1.size = '250,90'
    $GroupBox1.text = "Export options:"
    
    $RadioButton1 = New-Object System.Windows.Forms.RadioButton
    $RadioButton1.Location = '10,20'
    $RadioButton1.size = '180,20'
    $RadioButton1.Checked = $true 
    $RadioButton1.Text = "Default"
 
    $RadioButton2 = New-Object System.Windows.Forms.RadioButton
    $RadioButton2.Location = '10,40'
    $RadioButton2.size = '230,20'
    $RadioButton2.Checked = $false
    $RadioButton2.Text = "Force the re-export of ALL settings"

    # Add all the GroupBox controls on one line
    $GroupBox1.Controls.AddRange(@($Radiobutton1,$RadioButton2))
    $form.Controls.Add($GroupBox1)

  }
  If ($Import) {
    $buttonText1 = "Import my settings"

    $GroupBox1 = New-Object System.Windows.Forms.GroupBox
    $GroupBox1.Location = '40,110'
    $GroupBox1.size = '250,90'
    $GroupBox1.text = "Import options:"
    
    $RadioButton1 = New-Object System.Windows.Forms.RadioButton
    $RadioButton1.Location = '10,20'
    $RadioButton1.size = '180,20'
    $RadioButton1.Checked = $true 
    $RadioButton1.Text = "Default"
 
    $RadioButton2 = New-Object System.Windows.Forms.RadioButton
    $RadioButton2.Location = '10,40'
    $RadioButton2.size = '230,20'
    $RadioButton2.Checked = $false
    $RadioButton2.Text = "Force the re-import of ALL settings"

    $RadioButton3 = New-Object System.Windows.Forms.RadioButton
    $RadioButton3.Location = '10,60'
    $RadioButton3.size = '230,20'
    $RadioButton3.Checked = $false
    $RadioButton3.Text = "Force the import of Mapped Drives ONLY"

    # Add all the GroupBox controls on one line
    $GroupBox1.Controls.AddRange(@($Radiobutton1,$RadioButton2,$RadioButton3))
    $form.Controls.Add($GroupBox1)
  }

  $action1Button.Text = $buttonText1

  $action1Button.Add_Click({
    $ButtonType = [System.Windows.MessageBoxButton]::OK
    $MessageIcon = [System.Windows.MessageBoxImage]::Information

    If ($Export) {
      If ($RadioButton1.Checked -AND (!($RadioButton2.Checked))) {
        $ProcessAllIniFiles = $True
        $IgnoreFlag = $False
        $DriveMappingsOnly = $False
      }
      If ($RadioButton2.Checked -AND (!($RadioButton1.Checked))) {
        $ProcessAllIniFiles = $True
        $IgnoreFlag = $True
        $DriveMappingsOnly = $False
      }
    }

    If ($Import) {
      If ($RadioButton1.Checked -AND (!($RadioButton2.Checked)) -AND (!($RadioButton3.Checked))) {
        $ProcessAllIniFiles = $True
        $IgnoreFlag = $False
        $DriveMappingsOnly = $False
      }
      If ($RadioButton2.Checked -AND (!($RadioButton1.Checked)) -AND (!($RadioButton3.Checked))) {
        $ProcessAllIniFiles = $True
        $IgnoreFlag = $True
        $DriveMappingsOnly = $False
      }
      If ($RadioButton3.Checked  -AND (!($RadioButton1.Checked)) -AND (!($RadioButton2.Checked))) {
        $ProcessAllIniFiles = $False
        $IgnoreFlag = $True
        $DriveMappingsOnly = $True
      }
    }

    Call-Main -Import:$Import -Export:$Export -ProcessAllIniFiles:$ProcessAllIniFiles -ShowUI:$ShowUI -DriveMappingsOnly:$DriveMappingsOnly -IgnoreFlag:$IgnoreFlag

  })

  $form.AcceptButton = $action1Button
  $form.Controls.Add($action1Button)

  #-------------------------------------------------------------

  $form.Add_Resize({
    #$userTxtBox.Size = [System.Drawing.Size]::new(($this.Width-120),60)
    #$passwordTxtBox.Size = [System.Drawing.Size]::new(($this.Width-120),60)
  })

  $form.Topmost = $true
  $form.Add_Shown({$action1Button.Select()})
  $form.ShowDialog() | out-null #This starts the GUI
}

#------------------------------------

# This starts the functions depending on whether or not the UI is required.
If ($ShowUI -eq $False) {
  Call-Main -Import:$Import -Export:$Export -ProcessAllIniFiles:$ProcessAllIniFiles -ShowUI:$ShowUI
} Else {
  Call-MainForm -Import:$Import -Export:$Export -ProcessAllIniFiles:$ProcessAllIniFiles -ShowUI:$ShowUI
}

#------------------------------------

$EndDTM = (Get-Date)
Write-verbose "Elapsed Time: $(($EndDTM-$StartDTM).TotalSeconds) Seconds" -Verbose
Write-Verbose "Elapsed Time: $(($EndDTM-$StartDTM).TotalMinutes) Minutes" -Verbose

try {
  Stop-Transcript
}
catch {
  Write-Verbose "$(Get-Date): This host does not support transcription"
}
